""" Unittesting """
import unittest
from pathlib import Path
from datetime import datetime

from piwaterflow import Waterflow


class Testing(unittest.TestCase):
    """ Unittesting class
    """
    def setUp(self):
        template_config_path = f'{Path(__file__).parent}/data/config-template_reverse.yml'
        self.waterflow = Waterflow(template_config_path=template_config_path, dry_run=True)

    def test_0021_recalc_next_program(self):
        """ Test the recalc_next_program function
        """
        # Checks if _recalc_next_program works fine if last_program was "yesterday"
        last_program_time_utc = Waterflow.str_to_time('2023-04-26 23:30:27')
        fake_now = Waterflow.str_to_time('2023-04-27 00:01:00')
        time_utc, program_name = self.waterflow._recalc_next_program(last_program_time_utc, fake_now) # pylint: disable=protected-access
        self.assertTrue(time_utc == datetime(2023, 4, 27, 5, 6).astimezone())
        self.assertTrue(program_name == 'second')

        # Checks if _recalc_next_program works fine if last_program is today before first program
        last_program_time_utc = Waterflow.str_to_time('2023-04-27 00:30:00') # pylint: disable=protected-access
        time_utc, program_name = self.waterflow._recalc_next_program(last_program_time_utc, fake_now) # pylint: disable=protected-access
        self.assertTrue(time_utc == datetime(2023, 4, 27, 5, 6).astimezone())
        self.assertTrue(program_name == 'second')

        # Checks if _recalc_next_program works fine if last_program is between two programs
        last_program_time_utc = Waterflow.str_to_time('2023-04-27 10:30:00') # pylint: disable=protected-access
        time_utc, program_name = self.waterflow._recalc_next_program(last_program_time_utc, fake_now) # pylint: disable=protected-access
        self.assertTrue(time_utc == datetime(2023, 4, 27, 23, 30).astimezone())
        self.assertTrue(program_name == 'first')

        # Checks if _recalc_next_program works fine if last_program is after all  programs
        last_program_time_utc = Waterflow.str_to_time('2023-04-27 23:59:00') # pylint: disable=protected-access
        time_utc, program_name = self.waterflow._recalc_next_program(last_program_time_utc, fake_now) # pylint: disable=protected-access
        self.assertTrue(time_utc == datetime(2023, 4, 28, 5, 6,).astimezone())
        self.assertTrue(program_name == 'second')

if __name__ == '__main__':
    unittest.main()



""" Unittesting """
import unittest
from pathlib import Path

from piwaterflow import Waterflow


class Testing(unittest.TestCase):
    """ Unittesting class
    """
    def setUp(self):
        template_config_path = f'{Path(__file__).parent}/data/config-template.yml'
        self.waterflow = Waterflow(template_config_path=template_config_path, dry_run=True)

    def test_0000_loop(self):
        """ Test the loop and main high-level functionalities
        """
        self.waterflow.loop()

if __name__ == '__main__':
    unittest.main()



""" Unittesting """
import os
import time
import unittest
from pathlib import Path

from piwaterflow import Waterflow


class Testing(unittest.TestCase):
    """ Unittesting class
    """
    def setUp(self):
        template_config_path = f'{Path(__file__).parent}/data/config-template.yml'
        self.waterflow = Waterflow(template_config_path=template_config_path, dry_run=True)

    def test_0030_loop(self):
        """ Test the loop and main high-level functionalities
        """
        first_lock = self.waterflow.get_lock()
        self.assertTrue(first_lock)

        second_lock = self.waterflow.get_lock()
        self.assertFalse(second_lock)

        # Simulate lock expiring
        lock_path = self.waterflow._get_homevar_path('lock') # pylint: disable=protected-access
        past_epoch = time.time() - 1260
        os.utime(lock_path, (past_epoch, past_epoch) ) # 21 minutes before to simulate expiring
        third_lock = self.waterflow.get_lock()
        self.assertTrue(third_lock)


if __name__ == '__main__':
    unittest.main()



""" Config override for piwaterflow """
from datetime import datetime
import pytz

from config_yml import Config

def _set_timezone_utc(date: datetime):
    return pytz.timezone('UTC').localize(date)


class WaterflowConfig(Config):
    """Config override for piwaterflow
    """
    def _after_reading(self):
        """ Adapt the data after reading the config yaml
        """
        # Customize values
        for program in self.config['programs'].values():
            progtime = datetime.strptime(program['start_time'], '%H:%M')
            progtime = _set_timezone_utc(progtime)
            program['start_time'] = progtime

        # Sort the programs by time
        # self.config['programs'].sort(key=lambda prog: prog['start_time'])

    def _before_writting(self):
        """ Transforms the data before being written to the config file
        Returns:
            dict: Transformed config dictionary
        """
        configcopy = self.get_dict_copy()
        # Convert the date back from datetime to string
        for program in configcopy['programs'].values():
            program['start_time'] = program['start_time'].strftime('%H:%M')

        return configcopy
